package ANNs;
/**
 * @author Orisun
 * date 2011-10-20
 */
import java.util.ArrayList;
import java.util.Random;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

public class Controller {
	
	private static Display display;
	private static Shell shell;
	private Canvas canvas;
	
	private int num_tanks=Params.num_tank;	//坦克的数量
	private int num_mines=Params.num_mine;	//地雷的数量
	ArrayList<Tank> tanks;	//存储所有的坦克信息
	ArrayList<Mine> mines;	//存储所有的地雷信息
	
	private ArrayList<NeuralNet> neuralNets;	//每辆坦克都有一个神经网络在控制它

	int ticks=Params.Ticks;//按照神经网络移动ticks次后，使用遗传算法对神经网络的权重参数进行一次优化
	private boolean pause = false;		//中断
	private boolean game_over=false;
	
	long start_time;	//游戏开始时间
	long end_time;		//游戏结束时间
	
	int marks=0;		//已经扫除的地雷数
	
	public Controller(){
		tanks= new ArrayList<Tank>(num_tanks);
		mines= new ArrayList<Mine>(num_mines);
		neuralNets=new ArrayList<NeuralNet>(num_tanks);
		try{
			for(int i=0;i<num_tanks;i++)
				neuralNets.add(new NeuralNet());
		}catch(Exception e){
			System.err.println("创建网络失败:"+e.getMessage());
		}
	}
	
	public void start() {
		display = new Display();
		
		shell = new Shell(display);
		shell.setText("Smart Minesweeper");
		shell.setImage(new Image(display, "tank.png"));
		shell.setBackground(display.getSystemColor(SWT.COLOR_WHITE));
		shell.setSize(650, 600);
		
		canvas=new Canvas(shell,SWT.NO_BACKGROUND|SWT.NO_REDRAW_RESIZE|SWT.NO_MERGE_PAINTS|SWT.DOUBLE_BUFFERED);
		int width=shell.getClientArea().width;
		int height=shell.getClientArea().height;
		//System.out.println("width="+width+" height="+height);
		if(mines.size()==0||tanks.size()==0)
			initTanksAndMines(width, height);		//实例化每一个地雷和坦克，给它们一个随机的位置
		canvas.setSize(width, height);
		
		start_time=System.currentTimeMillis();
		
		canvas.addPaintListener(new PaintListener(){
			//根据tanks和mines中存储的位置和颜色信息把地雷和坦克绘出来
			public void paintControl(PaintEvent event) {
				for (int i = 0; i < mines.size(); i++) {
					event.gc.setForeground(((Composite)event.getSource()).getDisplay().getSystemColor(mines.get(i).color));
					int x = mines.get(i).coordinate_x;
					int y = mines.get(i).coordinate_y;
					event.gc.drawRectangle(x - 3, y - 3, 6, 6);
				}
				for(int i=0;i<tanks.size();i++){
					Tank tank=tanks.get(i);
					event.gc.setForeground(((Composite)event.getSource()).getDisplay().getSystemColor(tank.color));
					int center_x = tanks.get(i).coordinate_x;
					int center_y = tanks.get(i).coordinate_y;
					double angle=tank.angle;
					int a1=(int)(4*Math.sin(angle)+center_x);
					int a2=(int)(-4*Math.cos(angle)+center_y);
					int b1=(int)(12*Math.sin(angle)+center_x);
					int b2=(int)(-12*Math.cos(angle)+center_y);
					int c1=(int)(-16*Math.cos(angle)+12*Math.sin(angle)+center_x);
					int c2=(int)(-16*Math.sin(angle)-12*Math.cos(angle)+center_y);
					int d1=(int)(-16*Math.cos(angle)-12*Math.sin(angle)+center_x);
					int d2=(int)(-16*Math.sin(angle)+12*Math.cos(angle)+center_y);
					int e1=(int)(-12*Math.sin(angle)+center_x);
					int e2=(int)(12*Math.cos(angle)+center_y);
					int f1=(int)(-4*Math.sin(angle)+center_x);
					int f2=(int)(4*Math.cos(angle)+center_y);
					int g1=(int)(-16*Math.cos(angle)-4*Math.sin(angle)+center_x);
					int g2=(int)(-16*Math.sin(angle)+4*Math.cos(angle)+center_y);
					int h1=(int)(10*Math.cos(angle)-4*Math.sin(angle)+center_x);
					int h2=(int)(10*Math.sin(angle)+4*Math.cos(angle)+center_y);
					int q1=(int)(10*Math.cos(angle)+4*Math.sin(angle)+center_x);
					int q2=(int)(10*Math.sin(angle)-4*Math.cos(angle)+center_y);
					int t1=(int)(-16*Math.cos(angle)+4*Math.sin(angle)+center_x);
					int t2=(int)(-16*Math.sin(angle)-4*Math.cos(angle)+center_y);
					event.gc.drawPolyline(new int[]{a1,a2,b1,b2,c1,c2,d1,d2,e1,e2,f1,f2});
					event.gc.drawPolyline(new int[]{g1,g2,h1,h2,q1,q2,t1,t2});
				}
				if(game_over==true){
					GC gc=new GC(canvas);
					Font font=new Font(Display.getDefault(),"Times",20,SWT.BOLD);
					gc.setFont(font);
					gc.setForeground(new Color(display,255,0,0));
					gc.drawText("Game Over! You Use Time "+(end_time-start_time)/1000+" Secondes!", 100, 200);
				}
			}
		});
		
		canvas.addMouseListener(new MouseAdapter(){
			public void mouseDown(MouseEvent arg0) {
				if(pause)
					pause=false;
				else
					pause=true;
			}
		});
		
		shell.open();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}
		display.dispose();
	}
	
	public void flush(){
		Runnable runnable = new Runnable() {
			public void run() {
				try{
					Thread.sleep(1000);	//先休息1秒,让start()先执行
				}catch(InterruptedException e){
					e.printStackTrace();
				}
				while (!game_over) {
					if(pause)
						continue;
					try {
						Thread.sleep(100);	//休息0.1秒
						if (ticks > 0) {
							ticks--;
							updateMap();
							//System.out.println("在后台更新了一次地雷坦克信息");
						} else {
							//刷新ticks帧后，使用遗传算法对神经网络的权重参数进行一次优化
							gaOptimization();
							ticks=Params.Ticks;
							//b_update = false;
						}
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
					display.asyncExec(new Runnable() {
						public void run() {
							canvas.redraw();
							//System.out.println("调用地图的redraw.");
						}
					});
				}
			}
		};
		// 建立后台线程，持续更新坦克和地雷的信息
		Thread t = new Thread(runnable);
		t.setDaemon(true);
		t.start();
	}

	public static void main(String[] args) {
		Controller controller=new Controller();
		
		controller.flush();
		controller.start();
	}
	
	//使用神经网络，更新地雷和坦克的位置、颜色、速度、视角等信息
	public void updateMap(){
		for(int i=0;i<num_tanks;i++){
			Tank tank=tanks.get(i);
			//找到离坦克最近的地雷
			Mine nearMine=findNearMine(tank);
			if(nearMine==null){
				System.out.println("已经扫完所有地雷");
				//b_update=false;
				end_time=System.currentTimeMillis();
				game_over=true;
				return;
			}
			//算出神经网络的4个输入值
			double i1=nearMine.coordinate_x-tank.coordinate_x;
			double i2=nearMine.coordinate_y-tank.coordinate_y;
			unitVec(i1,i2);
			double i3=tank.speed*Math.cos(tank.angle);
			double i4=tank.speed*Math.sin(tank.angle);
			//输入到神经网络得到输出
			ArrayList<Double> input=new ArrayList<Double>(4);
			input.add(i1);
			input.add(i2);
			input.add(i3);
			input.add(i4);

			ArrayList<Double> output=neuralNets.get(i).getOutput(input);
			if(output.size()!=2){
				System.out.println("发生错误，神经网络的输出层不是2");
				return;
			}
			double rotForce=output.get(0)-output.get(1);
			double speed=output.get(0)+output.get(1);
			//更新坦克的angle和speed
			if(rotForce>Params.dMaxTurnRate)
				tank.angle+=Params.dMaxTurnRate;
			else if(rotForce<0-Params.dMaxTurnRate)
				tank.angle-=Params.dMaxTurnRate;
			else
				tank.angle+=rotForce;
			if(tank.angle>2*Math.PI||tank.angle==2*Math.PI){
				tank.angle-=2*Math.PI;
			}
			tank.speed=speed;
//			//计算坦克和最近地雷之间的距离
//			double distance=calEuDist(nearMine.coordinate_x, nearMine.coordinate_y, tank.coordinate_x, tank.coordinate_y);
//			distance=Math.sqrt(distance);
//			//坦克前进的步长为此距离的1/20
//			double step=distance/20;
			//更新坦克的坐标
			int increment_x=(int) (Params.step*tank.speed*Math.cos(tank.angle));
			int increment_y=(int) (Params.step*tank.speed*Math.sin(tank.angle));
			int newx=tank.coordinate_x+increment_x;
			int newy=tank.coordinate_y+increment_y;
			if(newx>644){
				tank.coordinate_x=640;
				if(tank.angle<Math.PI/2){
					tank.angle=Math.PI/2;
				}
				else if(tank.angle>3*Math.PI/2){
					tank.angle=3*Math.PI/2;
				}
				else
					tank.angle+=Math.PI;
			}
			else if(newx<0){
				tank.coordinate_x=5;
				if(tank.angle<Math.PI){
					tank.angle=Math.PI/2;
				}
				else if(tank.angle>Math.PI){
					tank.angle=3*Math.PI/2;
				}
				else
					tank.angle+=Math.PI;
			}
			else
				tank.coordinate_x=newx;
			if(newy>571){
				tank.coordinate_y=565;
				if(tank.angle<3*Math.PI/2){
					tank.angle=Math.PI;
				}
				else if(tank.angle>3*Math.PI/2){
					tank.angle=0;
				}
				else
					tank.angle+=Math.PI;	
			}
			else if(newy<0){
				tank.coordinate_y=5;
				if(tank.angle<Math.PI/2){
					tank.angle=0;
				}
				else if(tank.angle>Math.PI/2){
					tank.angle=Math.PI;
				}
				else
					tank.angle+=Math.PI;	
			}
			else
				tank.coordinate_y=newy;
			//如果地雷在坦克邻近区域内，则将它的颜色改为绿色
			if(calEuDist(nearMine.coordinate_x, nearMine.coordinate_y, tank.coordinate_x, tank.coordinate_y)<130){
				nearMine.color=SWT.COLOR_GREEN;
				tank.fitness++;	//每扫除一个地雷坦克的适应值就加1
				marks++;
				//当只剩下3个地雷时加大使用遗传算法的频率
				if(Params.num_mine-marks==3)
					Params.Ticks=Params.Ticks/4;
			}
		}
	}
	
	//单位化向量
	public void unitVec(double x,double y){
		double dd=Math.pow(x, 2)+Math.pow(y, 2);
		x/=Math.sqrt(dd);
		y/=Math.sqrt(dd);
	}
	
	//找到离tank最近的地雷
	public Mine findNearMine(Tank tank){
		int nearIndex=-1;
		int neardist=Integer.MAX_VALUE;
		int x1=tank.coordinate_x;
		int y1=tank.coordinate_y;
		for(int i=0;i<mines.size();i++){
			Mine mine=mines.get(i);
			//略过绿色的地雷，因为它已经被扫除了
			if(mine.color==SWT.COLOR_GREEN)
				continue;
			int dist=calEuDist(x1,y1,mine.coordinate_x,mine.coordinate_y);
			if(dist<neardist){
				neardist=dist;
				nearIndex=i;
			}
		}
		if(nearIndex==-1)
			return null;
		return mines.get(nearIndex);
	}
	
	//计算二维平面上两个点欧氏距离的平方
	public int calEuDist(int x1,int y1,int x2,int y2){
		return (int) (Math.pow(x1-x2, 2)+ Math.pow(y1-y2, 2));
	}
	
	//实例化每一个地雷和坦克，给它们一个随机的位置
	public void initTanksAndMines(int width, int height) {
		Random rand = new Random(System.currentTimeMillis());
		for (int i = 0; i < num_tanks; i++)
			tanks.add(new Tank(width, height, rand));
		for (int i = 0; i < num_mines; i++)
			mines.add(new Mine(width, height, rand));
	}
	
	//使用遗传算法对神经网络进行优化
	public void gaOptimization(){
		int genome_len=0;
		//收集所有神经网络的权重参数，以及对应坦克的适应值，以此构造Genome实例
		ArrayList<Genome> individuals=new ArrayList<Genome>(num_tanks);
		for(int i=0;i<num_tanks;i++){
			ArrayList<Double> weights=neuralNets.get(i).getWeights();
			genome_len=weights.size();
			double fitness=tanks.get(i).fitness;
			Genome genome=new Genome(weights,fitness);
			individuals.add(genome);
		}
		//得到GA优化之后的权重值
		GA ga=new GA(genome_len,num_tanks);
		ArrayList<ArrayList<Double>> children=ga.run(individuals);
		//把新的权重值再赋给神经网络
		
		for(int i=0;i<num_tanks;i++){
			ArrayList<Double> child=children.get(i);
			neuralNets.get(i).putWeights(child);
		}
	}
}
