package ANNs;
/**
 * @author Orisun
 * date 2011-10-20
 */
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
public class NeuralNet {

	int numInputs; // 神经网络输入层有多少个细胞
	int numOutputs; // 神经网络输出层有多少个细胞
	int numHiddenLayers; // 神经网络有几个隐藏层
	int numNeuronsPerHiddenLayer; // 每个隐藏层有多少个细胞

	ArrayList<NeuronLayer> arrLayers; // 存储神经网络的每一层

	// 构造函数。读取参数文件来初始化所有的Private成员变量，然后再调用CreateNet来创建网络
	public NeuralNet() throws Exception {
		File file = new File("src/ANNs/ANNparams");
		if (!file.exists()) {
			throw new Exception("参数文件不存在");
		} else {
			try {
				FileReader fr = new FileReader(file);
				BufferedReader br = new BufferedReader(fr);
				String line;
				while ((line = br.readLine()) != null) {
					if (line.trim().startsWith("#"))
						continue;
					String[] content = line.split("=");
					if (content[0].equals("numInputs"))
						numInputs = Integer.parseInt(content[1]);
					else if (content[0].equals("numOutputs"))
						numOutputs = Integer.parseInt(content[1]);
					else if (content[0].equals("numHiddenLayers"))
						numHiddenLayers = Integer.parseInt(content[1]);
					else if (content[0].equals("numNeuronsPerHiddenLayer"))
						numNeuronsPerHiddenLayer = Integer.parseInt(content[1]);
				}
				br.close();
			} catch (IOException e) {
				// System.out.println("参数文件被损坏" + e.getMessage());
				throw new Exception("参数文件被损坏" + e.getMessage());
			}
			
			try {
				createNet();
			} catch (Exception e) {
				System.out.println("CreateNet Fail.");
				e.printStackTrace();
			}
		}
	}

	// 创建网络(包括给所有的权重赋初始值)
	public void createNet() {
		arrLayers=new ArrayList<NeuronLayer>(numHiddenLayers+1);
		if (numHiddenLayers > 0) {
			// 创建第一层隐藏层
			arrLayers.add(new NeuronLayer(numNeuronsPerHiddenLayer, numInputs));
			//System.out.println("已创建第一层隐藏层");
			// 创建其他隐藏层
			for (int i = 1; i < numHiddenLayers; i++) {
				arrLayers.add(new NeuronLayer(numNeuronsPerHiddenLayer,
						numNeuronsPerHiddenLayer));
			}
			// 创建输出层
			arrLayers
					.add(new NeuronLayer(numOutputs, numNeuronsPerHiddenLayer));
			//System.out.println("已创建输出层");
		}
		// 无隐藏层时只创建输出层
		else {
			arrLayers.add(new NeuronLayer(numOutputs, numInputs));
		}
	}

	// 根据神经网络的一组输入计算输出
	public ArrayList<Double> getOutput(ArrayList<Double> inputs) {
		//System.out.println("numInputs="+numInputs+"  numOutputs="+numOutputs+"  numHiddenLayers="+numHiddenLayers+"  numNeuronsPerHiddenLayer="+numNeuronsPerHiddenLayer);
		ArrayList<Double> outputs = new ArrayList<Double>(); // 保存每一层产生的输出
		// 检查输入的个数是否正确
		if (inputs.size() != numInputs) {
			return outputs;
		}
		int index = 0;
		// 对于网络的每一层都计算它们的输出。隐藏层加输出层，网络一共有numHiddenLayers+1层
		for (int i = 0; i < numHiddenLayers + 1; ++i) {
			//System.out.println("i="+i);
			if (i > 0){
				inputs.clear();
				for(double d:outputs)
					inputs.add(d); // 用上一层的输出作为本层的输入
			}
			outputs.clear(); // 清空输出
			//System.out.println("本层输入有"+inputs.size()+"个");
			// 对于本层的每一个细胞计算它的输出
			for (int j = 0; j < arrLayers.get(i).numNeurons; ++j) {
				//System.out.println("j="+j);
				index = 0;
				double sumInput = 0.0; // 存储当前细胞的加权输入和
				int num_Inputs = arrLayers.get(i).arrNeurons.get(j).numInputs; // 获取当前细胞的输入个数
				//System.out.println("num_Inputs="+num_Inputs);
				for (int k = 0; k < num_Inputs - 1; ++k) {
					//System.out.println("k="+k);
					//System.out.println("index="+index);
					sumInput += arrLayers.get(i).arrNeurons.get(j).arrWeight
							.get(k) * inputs.get(index++);
				}
				// 加入偏移值
				sumInput += arrLayers.get(i).arrNeurons.get(j).arrWeight
						.get(num_Inputs - 1) * Params.dBias;
				// 加权输入经过S形函数运算后得到本细胞的输出
				outputs.add(Sigmoid(sumInput, Params.dActivationResponse));
			}
		}
		return outputs;
	}

	// 从神经网络得到所有细胞的输入权重
	public ArrayList<Double> getWeights() {
		ArrayList<Double> weights = new ArrayList<Double>();
		for (NeuronLayer layer : arrLayers) {
			for (Neuron neuron : layer.arrNeurons) {
				for (double weight : neuron.arrWeight) {
					weights.add(weight);
				}
			}
		}
		return weights;
	}

	// 用新的权重代替原有的权重
	public void putWeights(ArrayList<Double> weights) {
		int index = 0;
		for (NeuronLayer layer : arrLayers) {
			for (Neuron neuron : layer.arrNeurons) {
				for (int k = 0; k < neuron.arrWeight.size(); k++) {
					neuron.arrWeight.set(k, weights.get(index++));
				}
			}
		}
	}

	// S型响应曲线
	public double Sigmoid(double activation, double response) {
		double out = -1 * activation / response;
		return 1.0 / (Math.pow(Math.E, out) + 1);
	}
}