package ANNs;
/**
 * @author Orisun
 * date 2011-10-20
 */
import java.util.ArrayList;
import java.util.Random;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;

public class WarMap extends Canvas{

	ArrayList<Tank> tanks= new ArrayList<Tank>(20);
	ArrayList<Mine> mines= new ArrayList<Mine>(50);
	
	public void initTanksAndMines(int width, int height) {
		Random rand = new Random(System.currentTimeMillis());
		for (int i = 0; i < 20; i++)
			tanks.add(new Tank(width, height, rand));
		for (int i = 0; i < 50; i++)
			mines.add(new Mine(width, height, rand));
	}

	public ArrayList<Tank> getTanks() {
		return tanks;
	}
	
	public void setTanks(ArrayList<Tank> tanks) {
		this.tanks = tanks;
	}

	public ArrayList<Mine> getMines() {
		return mines;
	}

	public void setMines(ArrayList<Mine> mines) {
		this.mines = mines;
	}
	
	public Mine findNearMine(Tank tank){
		int nearIndex=-1;
		int neardist=Integer.MAX_VALUE;
		int x1=tank.coordinate_x;
		int y1=tank.coordinate_y;
		for(int i=0;i<mines.size();i++){
			Mine mine=mines.get(i);
			//地雷被发现后标记为绿色
			if(mine.color==SWT.COLOR_GREEN)
				continue;
			int dist=calEuDist(x1,y1,mine.coordinate_x,mine.coordinate_y);
			if(dist<neardist){
				neardist=dist;
				nearIndex=i;
			}
		}
		if(nearIndex==-1)
			return null;
		return mines.get(nearIndex);
	}
	
	public static int calEuDist(int x1,int y1,int x2,int y2){
		return (int) (Math.pow(x1-x2, 2)+ Math.pow(y1-y2, 2));
	}

	public WarMap(Composite parent) {
		//避免闪屏
		super(parent, SWT.NO_BACKGROUND|SWT.NO_REDRAW_RESIZE|SWT.NO_MERGE_PAINTS|SWT.DOUBLE_BUFFERED);
		int width=parent.getClientArea().width;
		int height=parent.getClientArea().height;
		//System.out.println("width="+width+"  height="+height);
		if(mines.size()==0||tanks.size()==0)
			initTanksAndMines(width, height);
		this.setSize(width, height);
		
		this.addPaintListener(new PaintListener() {
			public void paintControl(PaintEvent event) {
				System.out.println("开始绘制地图.");
				for (int i = 0; i < mines.size(); i++) {
					event.gc.setForeground(((Composite)event.getSource()).getDisplay().getSystemColor(mines.get(i).color));
					int x = mines.get(i).coordinate_x;
					int y = mines.get(i).coordinate_y;
					event.gc.drawRectangle(x - 3, y - 3, 6, 6);
				}
				for(int i=0;i<tanks.size();i++){
					Tank tank=tanks.get(i);
					event.gc.setForeground(((Composite)event.getSource()).getDisplay().getSystemColor(tank.color));
					int center_x = tanks.get(i).coordinate_x;
					int center_y = tanks.get(i).coordinate_y;
					//System.out.println("x="+center_x+"  y="+center_y);
					double angle=tank.angle;
					int a1=(int)(4*Math.sin(angle)+center_x);
					int a2=(int)(-4*Math.cos(angle)+center_y);
					int b1=(int)(12*Math.sin(angle)+center_x);
					int b2=(int)(-12*Math.cos(angle)+center_y);
					int c1=(int)(-16*Math.cos(angle)+12*Math.sin(angle)+center_x);
					int c2=(int)(-16*Math.sin(angle)-12*Math.cos(angle)+center_y);
					int d1=(int)(-16*Math.cos(angle)-12*Math.sin(angle)+center_x);
					int d2=(int)(-16*Math.sin(angle)+12*Math.cos(angle)+center_y);
					int e1=(int)(-12*Math.sin(angle)+center_x);
					int e2=(int)(12*Math.cos(angle)+center_y);
					int f1=(int)(-4*Math.sin(angle)+center_x);
					int f2=(int)(4*Math.cos(angle)+center_y);
					int g1=(int)(-16*Math.cos(angle)-4*Math.sin(angle)+center_x);
					int g2=(int)(-16*Math.sin(angle)+4*Math.cos(angle)+center_y);
					int h1=(int)(10*Math.cos(angle)-4*Math.sin(angle)+center_x);
					int h2=(int)(10*Math.sin(angle)+4*Math.cos(angle)+center_y);
					int q1=(int)(10*Math.cos(angle)+4*Math.sin(angle)+center_x);
					int q2=(int)(10*Math.sin(angle)-4*Math.cos(angle)+center_y);
					int t1=(int)(-16*Math.cos(angle)+4*Math.sin(angle)+center_x);
					int t2=(int)(-16*Math.sin(angle)-4*Math.cos(angle)+center_y);
					event.gc.drawPolyline(new int[]{a1,a2,b1,b2,c1,c2,d1,d2,e1,e2,f1,f2});
					event.gc.drawPolyline(new int[]{g1,g2,h1,h2,q1,q2,t1,t2});
//					System.out.printf("%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\n",a1,a2,b1,b2,c1,c2,d1,d2,e1,e2,f1,f2);
//					System.out.printf("%d,%d,%d,%d,%d,%d,%d,%d\n",g1,g2,h1,h2,q1,q2,t1,t2);
				}
			}
		});
	}
}
